package departmentarium.ejb;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.NamingException;

import departmentarium.interfaces.ConfigurationLocal;
import departmentarium.interfaces.ConfigurationLocalHome;
import departmentarium.interfaces.ReportLocal;
import departmentarium.interfaces.ReportLocalHome;
import departmentarium.interfaces.UserLocal;
import departmentarium.interfaces.UserLocalHome;
import departmentarium.utils.*;
import departmentarium.utils.exceptions.ReportException;
import departmentarium.utils.exceptions.UserException;
import departmentarium.utils.wrappers.ConfigurationPojo;
import departmentarium.utils.wrappers.ReportDetails;
import departmentarium.utils.wrappers.UserPojo;

/**
 * @ejb.bean name="Departmentarium"
 *           display-name="Departmentarium Session Bean"
 *           jndi-name="ejb/Departmentarium"
 *           type="Stateless"
 *           view-type="remote"
 */
public class DepartmentariumBean implements SessionBean {

	private static final long serialVersionUID = -5955008185287459655L;

	private UserLocalHome myUserLocalHome;
	private ReportLocalHome myReportLocalHome;
	private ConfigurationLocalHome myConfigurationLocalHome;
	private UserLocal myUserLocal;
	/**
	 * Create method
	 * @ejb.create-method  view-type = "remote"
	 */
	public void ejbCreate() throws javax.ejb.CreateException {
		try {
			myUserLocalHome = EJBGetter.getUserLocalHome();
			myReportLocalHome = EJBGetter.getReportLocalHome();
			myConfigurationLocalHome = EJBGetter.getConfigurationLocalHome();
		} catch (NamingException e) {
			throw new CreateException("Error while looking up Home interfaces");
		}
	}

	public void setSessionContext(SessionContext ctx)
		throws EJBException,
		RemoteException {
	}

	public void ejbRemove() throws EJBException, RemoteException {
	}

	public void ejbActivate() throws EJBException, RemoteException {
	}

	public void ejbPassivate() throws EJBException, RemoteException {
	}

	/**
	 * Business method
	 * @throws FinderException
	 * @ejb.interface-method  view-type = "remote"
	 */
	public String getUserKind(String aUsername, String aPassword)
		throws FinderException {
		Collection methodCln = myUserLocalHome.findByLoginAndPassword(
			aUsername,
			aPassword);
		if (methodCln.size() == 1) {
			myUserLocal = (UserLocal) methodCln.iterator().next();
			if (myUserLocal.getIsAdmin())
				return Constants.ADMIN;
			else
				return Constants.USER;
		} else
			return Constants.NOT_VALID_USER;
	}

	private boolean isCurrentUserAdmin() {
		return (myUserLocal.getIsAdmin().booleanValue());
	}

	private ReportDetails reportDetailsFromReportLocal(ReportLocal aReportLocal)
		throws ReportException {
		return new ReportDetails(
			aReportLocal.getId(),
			aReportLocal.getTitle(),
			aReportLocal.getDate(),
			aReportLocal.getUser().getLogin(),
			aReportLocal.getKeywords(),
			aReportLocal.getIsACollection());
	}

	private void setCurrentUser(String aUsername, String aPassword)
		throws FinderException {
		Collection methodCln = myUserLocalHome.findByLoginAndPassword(
			aUsername,
			aPassword);
		if (methodCln.size() == 1)
			myUserLocal = (UserLocal) methodCln.iterator().next();
		else
			throw new FinderException(Constants.WRONG_PASSWORD);
	}

	/**
	 * Business method
	 * @throws CreateException
	 * @throws FinderException
	 * @throws UserException
	 * @ejb.interface-method  view-type = "remote"
	 */
	public void createUser(
		String aAdminName,
		String aAdminPassword,
		String aLogin,
		String aPassword)
		throws CreateException,
		FinderException,
		UserException {
		setCurrentUser(aAdminName, aAdminPassword);
		if (!isCurrentUserAdmin())
			throw new UserException(Constants.CANNOT_CREATE_USER);
		myUserLocalHome.create(aLogin, aPassword);
	}

	/**
	 * Business method
	 * @throws FinderException 
	 * @throws RemoveException 
	 * @throws UserException 
	 * @ejb.interface-method  view-type = "remote"
	 */
	public void removeUser(
		String aAdminName,
		String aAdminPassword,
		String aLogin) throws UserException, FinderException, RemoveException {
		setCurrentUser(aAdminName, aAdminPassword);
		if (!isCurrentUserAdmin())
			throw new UserException(Constants.CANNOT_REMOVE_USER);
		else {
			UserLocal userToRemove = myUserLocalHome.findByPrimaryKey(aLogin);
			try {
				userToRemove.remove();
			} catch (EJBException e) {
				e.printStackTrace();
			}
		}
	}
	/**
	 * Business method
	 * @throws CreateException 
	 * @ejb.interface-method  view-type = "remote"
	 */
	public void createAdmin(String aLogin, String aPassword)
		throws CreateException {
		myUserLocalHome.createAdmin(aLogin, aPassword);
	}

	/**
	 * Business method
	 * @throws FinderException 
	 * @throws UserException 
	 * @throws ReportException 
	 * @ejb.interface-method  view-type = "remote"
	 */
	public void updateReportForCurrentUser(
		String aUsername,
		String aPassword,
		ReportDetails aReportDetails,
		byte[] aFile) throws UserException, FinderException, ReportException {
		setCurrentUser(aUsername, aPassword);
		ReportLocal methodReportLocal = myReportLocalHome
			.findByPrimaryKey(aReportDetails.getId().toString());
		methodReportLocal.update(aReportDetails, aFile);
	}

	/**
	 * Business method
	 * @throws FinderException 
	 * @ejb.interface-method  view-type = "remote"
	 */
	public void addReport(
		String aUsername,
		String aPassword,
		ReportDetails aReportDetails,
		byte[] aFile) throws FinderException {
		setCurrentUser(aUsername, aPassword);
		if (!isReportIdCorrect(aReportDetails.getId()))
			throw new FinderException(Constants.REPORT_ID_NOT_AVAILABLE);
		else {
			try {
				myReportLocalHome.create(aReportDetails, aFile, myUserLocal);
			} catch (CreateException e) {
				e.printStackTrace();
			}
			myUserLocal.setObtainedId("");
		}
	}

	private boolean isReportIdCorrect(String id) {
		return (myUserLocal.getObtainedId().compareTo(id) == 0);
	}

	/**
	 * Business method
	 * @throws FinderException  
	 * @ejb.interface-method  view-type = "remote"
	 */
	public String getId(String aUsername, String aPassword)
		throws FinderException {
		setCurrentUser(aUsername, aPassword);
		String methodObtainedId = myUserLocal.getObtainedId();
		if (methodObtainedId == null || methodObtainedId.length() < 2)
			try {
				setCurrentId();
			} catch (FinderException e) {
				throw new FinderException(Constants.CONFIGURATION_NOT_PRESENT);
			}
		return myUserLocal.getObtainedId();
	}

	private void setCurrentId() throws FinderException {
		ConfigurationLocal methodConfigurationLocal = myConfigurationLocalHome
			.findByPrimaryKey(Constants.CONFIGURATION_PRIMARY_KEY);
		myUserLocal.setObtainedId(methodConfigurationLocal.provideReportId());
	}

	/**
	 * Business method
	 * @throws UserException 
	 * @throws FinderException 
	 * @throws RemoveException 
	 * @ejb.interface-method  view-type = "remote"
	 */
	public void removeReport(
		String aUsername,
		String aPassword,
		String aReportId)
		throws UserException,
		FinderException,
		RemoveException {
		setCurrentUser(aUsername, aPassword);
		ReportLocal reportLocalToRemove = myReportLocalHome
			.findByPrimaryKey(aReportId);
		try {
			reportLocalToRemove.remove();
		} catch (EJBException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Business method 
	 * @throws FinderException  
	 * @throws CreateException 
	 * @ejb.interface-method  view-type = "remote"
	 */
	public void configureDepartmentarium(
		String aAdminName,
		String aAdminPassword,
		ConfigurationPojo aConfigurationPojo)
		throws CreateException,
		FinderException {
		setCurrentUser(aAdminName, aAdminPassword);
		if (!isCurrentUserAdmin())
			throw new CreateException(Constants.CANNOT_CREATE_CONFIGURATION);
		try {
			ConfigurationLocal methodConfigurationLocal = myConfigurationLocalHome
				.findByPrimaryKey(Constants.CONFIGURATION_PRIMARY_KEY);
			methodConfigurationLocal.setName(aConfigurationPojo
				.getDepartmentariumName());
			methodConfigurationLocal.setHost(aConfigurationPojo
				.getDepartmentariumHost());
			methodConfigurationLocal.setWebsiteName(aConfigurationPojo
				.getDepartmentariumWebsiteName());
			methodConfigurationLocal.setWebServiceName(aConfigurationPojo
				.getDepartmentariumWebServiceName());
			methodConfigurationLocal.setAtenariumUrl(aConfigurationPojo
				.getAtenariumUrl());
			methodConfigurationLocal.setPrefix(aConfigurationPojo.getPrefix());
		} catch (FinderException e) {
			myConfigurationLocalHome.create(aConfigurationPojo
				.getDepartmentariumName(), aConfigurationPojo
				.getDepartmentariumHost(), aConfigurationPojo
				.getDepartmentariumWebsiteName(), aConfigurationPojo
				.getDepartmentariumWebServiceName(), aConfigurationPojo
				.getAtenariumUrl(), aConfigurationPojo.getPrefix());
		}
		WebServiceManager.contactAtenarium(
			Constants.ATENARIUM_REGISTRATION,
			aConfigurationPojo);
	}

	/**
	 * Business method
	 * @throws FinderException 
	 * @throws ReportException 
	 * @ejb.interface-method  view-type = "remote"
	 */
public ReportDetails[] getUserReportsDetails(String aUsername, String aPassword) throws FinderException, ReportException{
		setCurrentUser(aUsername,aPassword);
		Collection<ReportLocal> methodReportLocalsCollection = myUserLocal.getReports();
		ReportDetails[] methodReportDetailsArray = new ReportDetails[methodReportLocalsCollection.size()];
		Iterator<ReportLocal> iter = methodReportLocalsCollection.iterator();
		int i=0;
		while(iter.hasNext()) {
			methodReportDetailsArray[i] = reportDetailsFromReportLocal(iter.next());
			i++;
		}
		return methodReportDetailsArray;
	}
	/**
	 * Business method
	 * @throws UserException 
	 * @throws FinderException 
	 * @ejb.interface-method  view-type = "remote"
	 */
public UserPojo[] getUsers(String aAdminName,
			String aAdminPassword) throws UserException, FinderException {
		if (!isCurrentUserAdmin())
			throw new UserException(Constants.CANNOT_GET_USERS);
		else {
			Collection<UserLocal> usersLocal = myUserLocalHome.findAll();
			UserPojo[] toReturn = new UserPojo[usersLocal.size()];
			Iterator<UserLocal> iter = usersLocal.iterator();
			int i = 0;
			while (iter.hasNext()) {
				toReturn[i] = new UserPojo(iter.next());
				i++;
			}
			return toReturn;
		}
	}
	/**
	 * Business method
	 * @throws FinderException
	 * @ejb.interface-method  view-type = "remote"
	 */
	public ConfigurationPojo getConfiguration() throws FinderException {
		ConfigurationLocal methodConfLocal = myConfigurationLocalHome
			.findByPrimaryKey(Constants.CONFIGURATION_PRIMARY_KEY);
		return new ConfigurationPojo(methodConfLocal);
	}
	/**
	 * Business method
	 * @throws FinderException 
	 * @ejb.interface-method  view-type = "remote"
	 */
	public byte[] getReport(String aReportId) throws FinderException {
		ReportLocal methodReportLocal = myReportLocalHome.findByPrimaryKey(aReportId);
		return methodReportLocal.getFile();
	}
	
	/**
	 * Business method
	 * @ejb.interface-method  view-type = "remote"
	 */
	public Collection<ReportDetails> getReportDetailsCollection() {
		Collection<ReportDetails> toReturn = new LinkedList<ReportDetails>();
		try {
			Collection<ReportLocal> methodReportLocalColl =  myReportLocalHome.findAll();
			for(ReportLocal eachReportLocal : methodReportLocalColl)
					toReturn.add(reportDetailsFromReportLocal(eachReportLocal));
		} catch (FinderException e) {
			e.printStackTrace();
		} catch (ReportException e) {
		e.printStackTrace();
		}
		return toReturn;
	}
}
